CONNECT "progserv:URep_Test.gdb";

CREATE TABLE Lookups (
       Id                   INTEGER NOT NULL,
       Teg                  VARCHAR(40) NOT NULL,
       Ival                 INTEGER NOT NULL,
       Item                 VARCHAR(80) NOT NULL,
       Comment              VARCHAR(80),
       CONSTRAINT PK__Lookups PRIMARY KEY (Id)
);

CREATE UNIQUE INDEX IX1_Lookups ON Lookups
(
       Teg,
       Ival
);



CREATE TABLE URepForm (
       Id                   INTEGER NOT NULL,
       Numm                 INTEGER NOT NULL,
       Name                 VARCHAR(250) NOT NULL,
       IdDestination        INTEGER NOT NULL,
       IdTypExec            INTEGER NOT NULL,
       Description          VARCHAR(250),
       Report               BLOB,
       CONSTRAINT PK__URepForm PRIMARY KEY (Id),
       CONSTRAINT Uq_URepForm_Name UNIQUE (Name),
       CONSTRAINT Uq_URepForm_Numm UNIQUE (Numm)
);

CREATE TABLE URepFrm2Qry (
       Id                   INTEGER NOT NULL,
       IdForm               INTEGER NOT NULL,
       Numm                 INTEGER NOT NULL,
       IdQuery              INTEGER,
       IdRemoteData         INTEGER,
       IsMaster             INTEGER NOT NULL,
       QueryShortName       CHAR(20) NOT NULL,
       CONSTRAINT PK__URepFrm2Qry PRIMARY KEY (Id)
);

CREATE  INDEX IX1_URepFrm2Qry ON URepFrm2Qry (IdForm, IdQuery, Numm);
CREATE  INDEX IX2_URepFrm2Qry ON URepFrm2Qry (IdForm, IdRemoteData, Numm) ;

ALTER TABLE URepFrm2Qry 
  ADD CONSTRAINT CK_URepFrm2Qry CHECK (((IdQuery is null) and (not(IdRemoteData is null))) or ((not(IdQuery is null)) and (IdRemoteData is null)));


CREATE TABLE URepGroup (
       Id                   INTEGER NOT NULL,
       GrpName              VARCHAR(250) NOT NULL,
       IdParent             INTEGER,
       Numm                 INTEGER NOT NULL,
       CONSTRAINT PK__URepGroup PRIMARY KEY (Id),
       CONSTRAINT Uq_URepGroup_GrpName UNIQUE (GrpName),
       CONSTRAINT Uq_URepGroup_Numm UNIQUE (Numm)
);


CREATE TABLE URepGrpBody (
       Id                   INTEGER NOT NULL,
       IdGroup              INTEGER NOT NULL,
       IdForm               INTEGER,
       IdScript             INTEGER,
       Numm                 INTEGER NOT NULL,
       CONSTRAINT PK__URepGrpBody PRIMARY KEY (Id)
);

CREATE INDEX IX1_URepGrpBody ON URepGrpBody (IdGroup, IdForm);

CREATE INDEX IX2_URepGrpBody ON URepGrpBody (IdGroup, IdScript);

ALTER TABLE URepGrpBody 
  ADD CONSTRAINT CK_URepGrpBody CHECK (((IdForm is null) and (not(IdScript is null))) or ((not(IdForm is null)) and (IdScript is null)));



CREATE TABLE URepHistoryLog (
       Id                   INTEGER NOT NULL,
       IdAction             INTEGER NOT NULL,
       ValueId              INTEGER NOT NULL,
       login                VARCHAR(30) NOT NULL,
       username             VARCHAR(30) NOT NULL,
       data                 DATE NOT NULL,
       Name                 VARCHAR(255),
       CONSTRAINT PK__URepHistoryLog PRIMARY KEY (Id),
       CONSTRAINT Uq_URepHistoryLog UNIQUE (data, IdAction, ValueId)
);

CREATE INDEX IX1_URepHistoryLog ON URepHistoryLog
(
       login
);


CREATE TABLE URepImports (
       Id                   INTEGER NOT NULL,
       Name                 VARCHAR(250) NOT NULL,
       Description          VARCHAR(250),
       Numm                 INTEGER NOT NULL,
       FileTemplate         VARCHAR(250),
       IdScrEng             INTEGER NOT NULL,
       SQLtext              BLOB SUB_TYPE 1,
       ScriptText           BLOB SUB_TYPE 1,
       CONSTRAINT PK__URepImports PRIMARY KEY (Id),
       CONSTRAINT Uq_URepImports_Numm UNIQUE  (Numm),
       CONSTRAINT Uq_URepImports_Name UNIQUE  (Name)
);


CREATE TABLE URepParam (
       Id                   INTEGER NOT NULL,
       IdQuery              INTEGER,
       IdRemoteData         INTEGER,
       Numm                 INTEGER NOT NULL,
       ParName              CHAR(40) NOT NULL,
       IdParTyp             INTEGER NOT NULL,
       ParLabel             CHAR(40) NOT NULL,
       SQL                  BLOB SUB_TYPE 1,
       Width                INTEGER,
       LookupField          VARCHAR(40),
       LookupDisplay        VARCHAR(40),
       LookupDisplayIndex   INTEGER,
       IdTypDefVal          INTEGER NOT NULL,
       DefSQL               BLOB SUB_TYPE 1,
       CONSTRAINT PK__URepParam PRIMARY KEY (Id)
);

CREATE INDEX IX1_URepParam ON URepParam (IdQuery, Numm);

CREATE INDEX IX2_URepParam ON URepParam (IdRemoteData, Numm);

ALTER TABLE URepParam 
  ADD CONSTRAINT CK_URepParam CHECK (((IdQuery is null) and (not(IdRemoteData is null))) or ((not(IdQuery is null)) and (IdRemoteData is null)));


CREATE TABLE URepQuery (
       Id                   INTEGER NOT NULL,
       Numm                 INTEGER NOT NULL,
       Name                 VARCHAR(250) NOT NULL,
       Description          VARCHAR(250),
       SQLtext              BLOB SUB_TYPE 1,
       CONSTRAINT PK__URepQuery PRIMARY KEY (Id),
       CONSTRAINT Uq_URepQuery_Numm UNIQUE (Numm),
       CONSTRAINT Uq_URepQuery_Name UNIQUE (Name)
);



CREATE TABLE URepRemoteData (
       Id                   INTEGER NOT NULL,
       Numm                 INTEGER NOT NULL,
       Name                 VARCHAR(250) NOT NULL,
       Description          VARCHAR(250),
       IdRemtServerType     INTEGER NOT NULL,
       ServerName           VARCHAR(250),
       ServerGUID           VARCHAR(250),
       ComputerName         VARCHAR(250),
       IdRemtProtocol       INTEGER,
       Port                 INTEGER,
       CONSTRAINT PK__URepRemoteData PRIMARY KEY (Id),
       CONSTRAINT Uq_URepRemoteData_Numm UNIQUE (Numm),
       CONSTRAINT Uq_URepRemoteData_Name UNIQUE (Name)
);


CREATE TABLE URepScript (
       Id                   INTEGER NOT NULL,
       Numm                 INTEGER NOT NULL,
       Name                 VARCHAR(250) NOT NULL,
       Description          VARCHAR(250),
       IdScrEng             INTEGER NOT NULL,
       ScriptText           BLOB SUB_TYPE 1,
       CONSTRAINT PK__URepScript PRIMARY KEY (Id),
       CONSTRAINT Uq_URepScript_Numm UNIQUE (Numm),
       CONSTRAINT Uq_URepScript_Name UNIQUE (Name)
);


CREATE TABLE URepUsers (
       Id                   INTEGER NOT NULL,
       IdTypObj             INTEGER NOT NULL,
       IdObj                INTEGER NOT NULL,
       LoginName            CHAR(40) NOT NULL,
       CONSTRAINT PK__URepUsers PRIMARY KEY (Id),
       CONSTRAINT Uq1_URepUsers UNIQUE (IdTypObj, IdObj, LoginName)
);


CREATE TABLE URepLogins (
	LoginName char (40) NOT NULL,
	FullName char (40) NOT NULL,
	CONSTRAINT PK_URepLogins PRIMARY KEY (LoginName)
);


/*  ADD FOREIGN KEY CONSTRAINT */
ALTER TABLE URepFrm2Qry
       ADD CONSTRAINT FK_URepFrm2Qry_IdForm
           FOREIGN KEY (IdForm)
                             REFERENCES URepForm;

ALTER TABLE URepFrm2Qry
       ADD CONSTRAINT FK_URepFrm2Qry_IdQuery
           FOREIGN KEY (IdQuery)
                             REFERENCES URepQuery;

ALTER TABLE URepFrm2Qry
       ADD CONSTRAINT FK_URepFrm2Qry_IdRemoteData
           FOREIGN KEY (IdRemoteData)
                             REFERENCES URepRemoteData;

ALTER TABLE URepGrpBody
       ADD CONSTRAINT FK_URepGrpBody_IdForm
           FOREIGN KEY (IdForm)
                             REFERENCES URepForm;

ALTER TABLE URepGrpBody
       ADD CONSTRAINT FK_URepGrpBody_IdGroup
           FOREIGN KEY (IdGroup)
                            REFERENCES URepGroup;

ALTER TABLE URepGrpBody
       ADD CONSTRAINT FK_URepGrpBody_IdScript
           FOREIGN KEY (IdScript)
                             REFERENCES URepScript;

ALTER TABLE URepParam
       ADD CONSTRAINT FK_URepParam_IdQuery
           FOREIGN KEY (IdQuery)
                             REFERENCES URepQuery;

ALTER TABLE URepParam
       ADD CONSTRAINT FK_URepParam_IdRemoteData
           FOREIGN KEY (IdRemoteData)
                             REFERENCES URepRemoteData;
