CONNECT "progserv:URep_Test.gdb";

SET TERM ^ ;

CREATE VIEW vwURepForm
(Id, Numm, Name, IdDestination, IdTypExec, Description)
AS
SELECT Id, Numm, Name, IdDestination, IdTypExec, Description
FROM URepForm
WHERE Id IN (SELECT IdObj FROM URepUsers
             WHERE (LoginName=USER or
                    LoginName in (SELECT RDB$RELATION_NAME
                                  FROM RDB$USER_PRIVILEGES
                                  WHERE RDB$USER=USER))
               AND IdTypObj = 2)
   OR NOT EXISTS (SELECT * FROM URepUsers
                  WHERE IdTypObj = 2 AND IdObj=URepForm.Id)
   OR USER='SYSDBA'
;
 ^

CREATE VIEW vwURepGroup
(Id, GrpName, IdParent, Numm)
AS
SELECT Id, GrpName, IdParent, Numm
FROM URepGroup
WHERE Id IN (SELECT IdObj FROM URepUsers
             WHERE (LoginName=USER or
                    LoginName in (SELECT RDB$RELATION_NAME
                                  FROM RDB$USER_PRIVILEGES
                                  WHERE RDB$USER=USER))
               AND IdTypObj = 1)
   OR NOT EXISTS (SELECT * FROM URepUsers
                  WHERE IdTypObj = 1 AND IdObj=URepGroup.Id)
   OR USER='SYSDBA'
;
 ^

CREATE VIEW vwURepScript
(Id, Numm, Name, Description, IdScrEng)
AS
SELECT Id, Numm, Name, Description, IdScrEng
FROM URepScript
WHERE Id IN (SELECT IdObj FROM URepUsers
             WHERE (LoginName=USER or
                    LoginName in (SELECT RDB$RELATION_NAME
                                  FROM RDB$USER_PRIVILEGES
                                  WHERE RDB$USER=USER))
               AND IdTypObj = 5)
   OR NOT EXISTS (SELECT * FROM URepUsers
                  WHERE IdTypObj = 5 AND IdObj=URepScript.Id)
   OR USER='SYSDBA'
;
 ^

CREATE VIEW vwURepImports
(Id, Numm, Name, FileTemplate, Description, IdScrEng)
AS
SELECT Id, Numm, Name, FileTemplate, Description, IdScrEng
FROM URepImports
WHERE Id IN (SELECT IdObj FROM URepUsers
             WHERE (LoginName=USER or
                    LoginName in (SELECT RDB$RELATION_NAME
                                  FROM RDB$USER_PRIVILEGES
                                  WHERE RDB$USER=USER))
               AND IdTypObj = 6)
   OR NOT EXISTS (SELECT * FROM URepUsers
                  WHERE IdTypObj = 6 AND IdObj=URepImports.Id)
   OR USER='SYSDBA'
;
 ^

/*  CREATE PROCEDURE to make UNION */

CREATE PROCEDURE vwURepObjects
returns (Id Integer, IdTypObj Integer, IdObj Integer, ObjName varchar(250), TypObjName varchar(80), Numm Integer)
as
begin

  for
  select 1000000+u.Id Id, 1 IdTypObj, u.Id IdObj, GrpName ObjName, l.Item TypObjName, Numm
  from URepGroup u, Lookups l
  where l.Teg='URepUsers_IdTypObj' and l.Ival=1
  into :Id, :IdTypObj, :IdObj, :ObjName, :TypObjName, :Numm
  do suspend;

  for
  select 2000000+u.Id, 2, u.Id, Name, l.Item, Numm
  from URepForm u, Lookups l
  where l.Teg='URepUsers_IdTypObj' and l.Ival=2
  into :Id, :IdTypObj, :IdObj, :ObjName, :TypObjName, :Numm
  do suspend;

  for
  select 5000000+u.Id, 5, u.Id, Name, l.Item, Numm
  from URepScript u, Lookups l
  where l.Teg='URepUsers_IdTypObj' and l.Ival=5
  into :Id, :IdTypObj, :IdObj, :ObjName, :TypObjName, :Numm
  do suspend;

  for
  select 6000000+u.Id, 6, u.Id, Name, l.Item, Numm
  from URepImports u, Lookups l
  where l.Teg='URepUsers_IdTypObj' and l.Ival=6
  into :Id, :IdTypObj, :IdObj, :ObjName, :TypObjName, :Numm
  do suspend;
end;
 ^


CREATE PROCEDURE vwURepLogins
returns (LoginName char(40), FullName char(40), IsRole integer)
AS
DECLARE VARIABLE FullName2 char(40);
begin
  for
  select distinct
    cast(RDB$USER as char(40)) LoginName,
    cast('USER:'||RDB$USER as char(40)) FullName,
    0 IsRole
  from RDB$USER_PRIVILEGES pv
  where not exists(select * from RDB$ROLES r
                   where r.RDB$ROLE_NAME=pv.RDB$USER)
    and RDB$USER_TYPE<>5                       /* without sp names ???*/
  into :LoginName, :FullName, :IsRole
  do 
  begin
    select FullName from URepLogins where LoginName=:LoginName
    into :FullName2;
    if (FullName2 is not NULL) then
    begin
      FullName = FullName2;
      FullName2 = NULL;
    end
    suspend;
  end

  for
  select
    cast(RDB$ROLE_NAME as char(40)) LoginName,
    cast('ROLE:'||RDB$ROLE_NAME as char(40)) FullName,
    1 IsRole
  from RDB$ROLES
  into :LoginName, :FullName, :IsRole
  do 
  begin
    select FullName from URepLogins where LoginName=:LoginName
    into :FullName2;
    if (FullName2 is not NULL) then
    begin
      FullName = FullName2;
      FullName2 = NULL;
    end
    suspend;
  end
end;
 ^


CREATE PROCEDURE vwURepRightsByUser
returns (Id Integer, IdTypObj Integer, IdObj Integer, ObjName varchar(250),
         TypObjName varchar(80), Numm Integer, LoginName char(40), IdRight Integer)
AS
begin
  for
  select Id, IdTypObj, IdObj, ObjName, TypObjName, Numm, LoginName,
   (select Id from URepUsers R
      where R.IdTypObj=O.IdTypObj and
            R.IdObj=O.IdObj and
            R.LoginName=L.LoginName) IdRight
  from vwURepObjects O, vwURepLogins L
  into :Id, :IdTypObj, :IdObj, :ObjName, :TypObjName, :Numm, :LoginName, :IdRight
  do suspend;
end;
 ^

