CONNECT "progserv:URep_Test.gdb";


SET TERM ^ ;


CREATE PROCEDURE spURepGetIsAdmin RETURNS (IsAdmin INTEGER) 
AS    
  DECLARE VARIABLE Cnt1 INTEGER;
  DECLARE VARIABLE CurUSER VARCHAR(40);
BEGIN
  SELECT Count(*)
  FROM RDB$USER_PRIVILEGES
  WHERE RDB$USER=USER
    and RDB$RELATION_NAME='UREP_ADMIN'
  INTO :Cnt1;

  SELECT USER FROM RDB$DATABASE
  INTO :CurUSER;

  IF ((Cnt1>0) or (CurUSER='SYSDBA')) THEN
    IsAdmin=1;
  ELSE
    IsAdmin=0;
END
 ^

CREATE PROCEDURE spURepFormDel (IdForm INTEGER)
AS
begin
  delete from URepForm where Id=:IdForm;
end
 ^

CREATE PROCEDURE spURepFormNumm RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+10 from URepForm
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^

CREATE PROCEDURE spURepFrm2QryNumm (IdForm INTEGER) RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+1 
  from URepFrm2Qry where IdForm=:IdForm
  into :Numm;

  if (Numm is NULL) then
    Numm = 1;
end
 ^

CREATE PROCEDURE spURepGrpDel (IdGrp INTEGER)
AS
begin
  delete from URepGroup where Id=:IdGrp;
end
 ^

CREATE PROCEDURE spURepGrpNumm RETURNS (Numm INTEGER)
AS
begin
  select Max(Numm)+10 from URepGroup
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^

CREATE PROCEDURE spURepImportsNumm RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+10 from URepImports
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^

CREATE PROCEDURE spURepQueryDel (IdQuery INTEGER)
AS
begin
  delete from URepParam where IdQuery=:IdQuery;
  delete from URepQuery where Id=:IdQuery;
end
 ^

CREATE PROCEDURE spURepQueryNumm RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+10 from URepQuery
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^

CREATE PROCEDURE spURepScriptDel (IdScript INTEGER)
AS
begin
  delete from URepScript where Id=:IdScript;
end
 ^

CREATE PROCEDURE spURepScriptNumm RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+10 from URepScript
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^

CREATE PROCEDURE spURepFormAdd (Name varchar(250), IdQuery INTEGER) 
RETURNS (IdForm INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
  DECLARE VARIABLE IdTypExec INTEGER;
begin
  if (IdQuery=0) then
    IdTypExec=1; /* QR */
  else
    IdTypExec=1000; /* table */

  Execute procedure spURepFormNumm RETURNING_VALUES :Numm;

  insert into URepForm (Numm, Name, IdDestination, IdTypExec)
    values (:Numm, :Name, 0, :IdTypExec);

  select max(Id) from URepForm
  into :IdForm;

  if (IdQuery<>0) then
    begin
    Execute procedure spURepFrm2QryNumm :IdForm RETURNING_VALUES :Numm;

    insert into URepFrm2Qry (IdForm, Numm, IdQuery, QueryShortName, IsMaster)
      values(:IdForm, :Numm, :IdQuery, 'Query'||Cast(:Numm as varchar(5)), 0);
    end
end
 ^


CREATE PROCEDURE spURepGrpAdd (GrpName varchar(250)) 
RETURNS (IdGrp INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
begin
  Execute procedure spURepGrpNumm RETURNING_VALUES :Numm;

  insert into URepGroup (GrpName, IdParent, Numm)
    values (:GrpName, NULL, :Numm);

  select max(Id) from URepGroup
  into :IdGrp;
end
 ^

CREATE PROCEDURE spURepQueryAdd (Name varchar(250), IdForm INTEGER) 
RETURNS (IdQuery INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
begin
  Execute procedure spURepQueryNumm RETURNING_VALUES :Numm;

  insert into URepQuery (Numm, Name)
    values (:Numm, :Name);
  
  select max(Id) from URepQuery
  into :IdQuery;

  if (IdForm<>0) then
    begin
    Execute procedure spURepFrm2QryNumm :IdForm RETURNING_VALUES :Numm;

    insert into URepFrm2Qry (IdForm, Numm, IdQuery, QueryShortName, IsMaster) 
      values (:IdForm, :Numm, :IdQuery, 'Query'||Cast(:Numm as varchar(5)), 0);
    end
end
 ^

CREATE PROCEDURE spURepScriptAdd (Name varchar(250))
RETURNS (IdScript INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
begin
  Execute procedure spURepScriptNumm RETURNING_VALUES :Numm;

  insert into URepScript (Numm, Name, IdScrEng)
    values (:Numm, :Name, 1);

  select max(Id) from URepScript
  into :IdScript;
end
 ^

CREATE PROCEDURE spURepParamNumm (IdQuery INTEGER, IdRemoteData INTEGER) 
RETURNS (Numm INTEGER)
AS
begin
  if ((IdQuery is not NULL) and (IdQuery>0)) then
    begin
    select max(Numm)+1 
    from URepParam 
    where IdQuery=:IdQuery
    into :Numm;

    if (Numm is NULL) then
      Numm = 1;
    end
  else
    begin
    select max(Numm)+1
    from URepParam 
    where IdRemoteData=:IdRemoteData
    into :Numm;

    if (Numm is NULL) then
      Numm = 1;
    end
end
 ^

CREATE PROCEDURE spURepParamAdd (paramName varchar(40), IdQuery INTEGER, IdRemoteData INTEGER)
RETURNS (IdParam INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
begin
  Execute procedure spURepParamNumm :IdQuery, :IdRemoteData RETURNING_VALUES :Numm;

  if (IdRemoteData=0) then IdRemoteData = NULL;
  if (IdQuery=0) then IdQuery = NULL;

  insert into URepParam (IdQuery, IdRemoteData, Numm, ParName, ParLabel, IdParTyp, IdTypDefVal)
    values (:IdQuery, :IdRemoteData, :Numm, :paramName, :paramName, 0, 0);

  select max(Id) from URepParam
  into :IdParam;
end
 ^

CREATE PROCEDURE spURepParamsCopy 
(OldIdQuery INTEGER, OldIdRemoteData INTEGER, OldIdParam INTEGER, 
 NewIdQuery INTEGER, NewIdRemoteData INTEGER)
AS
  DECLARE VARIABLE Numm         INTEGER;
  DECLARE VARIABLE ParName      varchar(40);
  DECLARE VARIABLE IdParTyp     INTEGER;
  DECLARE VARIABLE ParLabel     varchar(40);
  DECLARE VARIABLE SQL          varchar(4096);
  DECLARE VARIABLE Width        INTEGER;
  DECLARE VARIABLE LookupField  varchar(40);
  DECLARE VARIABLE LookupDisplay      varchar(40);
  DECLARE VARIABLE LookupDisplayIndex INTEGER;
  DECLARE VARIABLE IdTypDefVal        INTEGER;
  DECLARE VARIABLE DefSQL       varchar(4096);
  DECLARE VARIABLE cnt1 INTEGER;
begin
  if (OldIdQuery>0) then
  begin
    for
    select Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
           LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL
    from URepParam
    where IdQuery=:OldIdQuery
    into :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL
    do
    begin

      select count(*) from URepParam where Numm=:Numm and IdQuery=:OldIdQuery
      into :cnt1;
      while (cnt1>0) do
      begin
        Numm = Numm + 1;

        select count(*) from URepParam where Numm=:Numm and IdQuery=:OldIdQuery
        into :cnt1;
      end

      select count(*) from URepParam where ParName=:ParName and IdQuery=:OldIdQuery
      into :cnt1;
      while (cnt1>0) do
      begin
        ParName = ParName + '_';

        select count(*) from URepParam where ParName=:ParName and IdQuery=:OldIdQuery
        into :cnt1;
      end

      insert into URepParam
        (IdQuery, Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
         LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL)
      values
        (:NewIdQuery, :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL);
    end
  end


  if (OldIdRemoteData>0) then
  begin
    for
    select Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
           LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL
    from URepParam
    where IdRemoteData=:OldIdRemoteData
    into :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL
    do
    begin

      select count(*) from URepParam where Numm=:Numm and IdRemoteData=:OldIdRemoteData
      into :cnt1;
      while (cnt1>0) do
      begin
        Numm = Numm + 1;

        select count(*) from URepParam where Numm=:Numm and IdRemoteData=:OldIdRemoteData
        into :cnt1;
      end

      select count(*) from URepParam where ParName=:ParName and IdRemoteData=:OldIdRemoteData
      into :cnt1;
      while (cnt1>0) do
      begin
        ParName = ParName + '_';

        select count(*) from URepParam where ParName=:ParName and IdRemoteData=:OldIdRemoteData
        into :cnt1;
      end

      insert into URepParam
        (IdRemoteData, Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
         LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL)
      values
        (:NewIdRemoteData, :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL);
    end
  end

  else if (OldIdParam>0) then
  begin
    for
    select Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
           LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL
    from URepParam
    where Id=:OldIdParam
    into :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL
    do
    begin

      select count(*) from URepParam where Numm=:Numm and IdQuery=:OldIdQuery
      into :cnt1;
      while (cnt1>0) do
      begin
        Numm = Numm + 1;

        select count(*) from URepParam where Numm=:Numm and IdQuery=:OldIdQuery
        into :cnt1;
      end

      select count(*) from URepParam where ParName=:ParName and IdQuery=:OldIdQuery
      into :cnt1;
      while (cnt1>0) do
      begin
        ParName = ParName + '_';

        select count(*) from URepParam where ParName=:ParName and IdQuery=:OldIdQuery
        into :cnt1;
      end

      insert into URepParam
        (IdQuery, IdRemoteData, Numm, ParName, IdParTyp, ParLabel, SQL, Width, 
         LookupField, LookupDisplay, LookupDisplayIndex, IdTypDefVal, DefSQL)
      values
        (:NewIdQuery, :NewIdRemoteData, :Numm, :ParName, :IdParTyp, :ParLabel, :SQL, :Width, 
         :LookupField, :LookupDisplay, :LookupDisplayIndex, :IdTypDefVal, :DefSQL);
    end
  end
end
 ^

CREATE PROCEDURE spURepLogAction (IdAction INTEGER, ValueId INTEGER, Name varchar(250))
AS
begin
  insert into URepHistoryLog (IdAction, ValueId, Name, login, username, data)
    values (:IdAction, :ValueId, :Name, USER, USER, 'NOW');
end
 ^

CREATE PROCEDURE spURepRemoteDataDel (IdRemoteData INTEGER)
AS
begin
  delete from URepParam where IdRemoteData=:IdRemoteData;
  delete from URepRemoteData where Id=:IdRemoteData;
end
 ^

CREATE PROCEDURE spURepRemoteDataNumm RETURNS (Numm INTEGER)
AS
begin
  select max(Numm)+10 from URepRemoteData
  into :Numm;

  if (Numm is NULL) then
    Numm = 10;
end
 ^


CREATE PROCEDURE spURepRemoteDataAdd (Name varchar(250))
RETURNS (IdRemoteData INTEGER)
AS
  DECLARE VARIABLE Numm INTEGER;
begin
  Execute procedure spURepRemoteDataNumm RETURNING_VALUES :Numm;

  insert into URepRemoteData (Numm, Name, IdRemtServerType)
    values (:Numm, :Name, 1);

  select max(Id) from URepRemoteData
  into :IdRemoteData;
end
 ^
